analyzeQLCounts <- function(counts, design, totals=colSums(counts), norm.factors=1, offset=NULL, contrast=NULL, plot=FALSE, everything=FALSE)
# Runs a DE analysis on some counts using edgeR's quasi-likelihood framework (i.e. the
# QuasiSeq methods).	
#
# written by Aaron Lun
{
	# Setting up the counts.
	y<-DGEList(counts, lib.size=totals)
	y$samples$norm.factors<-norm.factors
	y$offset <- offset
	y<-estimateDisp(y, design)	
	fit<-glmQLFit(y, design, robust=TRUE)
	
	if (plot) { 
		plotBCV(y) 
		plotQLDisp(fit)
	}
	
	out<-glmQLFTest(fit, contrast=contrast)
	tab<-topTags(out, nrow(out))$table
	tab<-tab[order(as.integer(rownames(tab))),]
	if (!everything) { 
		return(tab)
	} else {
		return(list(y=y, fit=out))
	}
}

#################
# DESeq2 methods, only included for old times' sake.
# require(DESeq2)
# design2<-design[,-1,drop=FALSE]
# colnames(design2)<-paste("x", 1:ncol(design2), sep="")
# dds <- DESeqDataSetFromMatrix(countData = counts, colData = data.frame(design2),
#    design= as.formula(paste(c("~", paste(colnames(design2), collapse = "+")))))
# size.fac<-y$samples$norm.factors*y$samples$lib.size
# sizeFactors(dds)<-size.fac/mean(size.fac)
# dds <- estimateDispersions(dds)
# dds <- nbinomWaldTest(dds)
# res <- results(dds)
# tab<-data.frame(logFC=res$log2FoldChange, PValue=res$pvalue, FDR=res$FDR, row.names=rownames(res))
#################

resultDump <- function(regions, tab, cutoff=0.05, out="out.tsv") 
# Dumping all the output into a specified output file.
{
	sig<-tab$FDR <= cutoff
	sig.bins<-regions[sig]
	write.table(data.frame(chr=as.character(seqnames(sig.bins)), start=start(sig.bins),
		end=end(sig.bins), tab[sig,]),
		file=out, row.names=FALSE, quote=FALSE, sep="\t")
}

