assessChIP <- function(observed, known, tol=200, checkfc=TRUE) 
# We read in any tab-delimited file where the first three columns are 'chr', 'start', and 'end' of
# each predicted enriched region. We just check that the enriched sites fall within the specified
# range. The number of true positives, false positives and false negatives are reported. Both
# are reported as the number of regions rather than in terms of bases. All regions with centres
# within 'tolerance' of one another and in the same direction are merged.
#
# written by Aaron Lun
{
    obs<-read.table(observed, header=TRUE);
	if (checkfc) {
		up.o<-obs$logFC > 0
		if (is.null(up.o)) { stop("need a log-FC field in the table of observed sites") }
    }else { up.o<-rep(TRUE, nrow(obs))  }
	obranges<-GRanges(obs$chr, IRanges(obs$start, obs$end))
	if (!is.na(tol)) {
		out<-mergeWindows(obranges, sign=up.o, tol=tol)
		obranges<-out$region
		all.signs<-logical(length(obranges))
		all.signs[out$id]<-up.o
		up.o<-all.signs
	}
    
	# Reading in the known sites. Should have a 'logFC' field.
	kx<-read.table(known, header=TRUE);
	if (checkfc) {
		up.t<-kx$logFC > 0
		if (is.null(up.t)) { stop("need a log-FC field in the table of known sites") }
    } else { up.t<-rep(TRUE,nrow(kx)) }
	if (!nrow(kx)) { stop("no known sites to check against"); }
    kranges<-GRanges(kx[,1], IRanges(kx[,2], kx[,3]))
	if (is.null(kx$name)) { 
		kranges$name <- 1:length(kranges)
	} else {
		kranges$name <- kx$name
	}

	# Pulling out some states
	known.up <- kranges[up.t]
	known.down <- kranges[!up.t]
	u.olap<-findOverlaps(known.up, obranges[up.o])
	d.olap<-findOverlaps(known.down, obranges[!up.o])
	recall<-length(unique(known.up$name[queryHits(u.olap)]))+length(unique(known.down$name[queryHits(d.olap)]))
	overlapped<-length(unique(subjectHits(u.olap)))+length(unique(subjectHits(d.olap)))
	found<-length(obranges)

    return(list(overlap=overlapped, found=found, recall=recall));
}

