convertBamToBed<-function(bam, out, mode="all", dedup=FALSE, minq=0, cmd.only=FALSE, flags="")
# This function converts BAM files to BED files, with screening out of duplicates
# and/or removal of chromosomes that aren't nuclear or autosomal. If multiple BAM
# files are supplied, they are pooled into the single BED file.
#
# written by Aaron Lun
{
	stopifnot(length(out)==1L)
    if (dedup) { flags <- paste(flags, "-F 1024") }
	minq<-as.integer(minq+0.5)
	if (minq>0L) { flags<-paste(flags, "-q", minq) }
    filter<-""
    if (mode!="all") {
		filter<-paste0("| grep -E '", sub("\\$$", "\t", pruneChromosomes(NULL, mode=mode, pattern=TRUE)), "'") 
    }

    # Assembling the commands for each BAM file.
	all.cmds<-NULL
    for (x in bam) {
        cmds<-c('samtools view -b', flags, x, "| bedtools bamtobed -i stdin", filter, ">>", out)
		current<-paste(cmds, collapse=" ")
		all.cmds<-append(all.cmds, current)
    }
	if (cmd.only) { return(all.cmds) }
    
	# Running through all BAM files and pooling them.
	unlink(out)
	for (current in all.cmds) {
   	 	if (system(current)) { stop("BAM to BED conversion failed for ", x) }
	}

	# Returning the genome characteristics of the BAM files.
	stuff<-scanBamHeader(bam[1])[[1]]$targets
	for (x in bam) {
		other<-scanBamHeader(x)[[1]]$targets
		if (!identical(other, stuff)) { warning("chromosome properties in BAM file headers do not match") }
	}			
    return(invisible(stuff))
}

