plotAnno <- function(incoming, txdb, anno, fcol="blue", rcol="red", 
	xlab="Genomic position (bp)", ...)
# This plots annotation in the 'incoming' range, making a plot with multiple
# tracks for exons. If there are overlapping genes, then it will correctly
# pull out those genes. It also makes a couple of pretty plots.
{
	totest <- resize(incoming, width(incoming)*1.1, fix="center")
	curex <- GenomicFeatures::exonsBy(txdb, by="gene")
	exlap <- findOverlaps(totest, curex)
	used.ex <- names(curex)[subjectHits(exlap)]

	# Going through all discovered genes and pulling out max/min, to determine
	# whether it should be on a separate track.
	ntracks <- ptracks <- list(IRanges())
	use.track <- NULL
	middle.earth <- NULL
	for (x in used.ex) {
		curx <- curex[[x]]
		isforward <- all(strand(curx)=="+")
		newrange <- IRanges(min(start(curx)), max(end(curx)))
		
		if (isforward) {
			curtracks <- ptracks
		} else {
			curtracks <- ntracks
		}
		isdone <- FALSE
		for (y in 1:length(curtracks)) {			
			if (!overlapsAny(newrange, curtracks[[y]])) { 
				isdone <- TRUE
				curtracks[[y]] <- c(curtracks[[y]], newrange)
				break 					
			}
		}
		if (!isdone) { curtracks[[length(curtracks)+1L]] <- newrange }
		if (isforward) { 
			ptracks <- curtracks
		} else {
			ntracks <- curtracks
		}
		use.track <- append(use.track, y)

		# Picking the location of the centred text.
		included <- IRanges::intersect(newrange, ranges(incoming))
		if (length(included)) { 
			middle.earth <- append(middle.earth, mid(included))
		} else {
			middle.earth <- append(middle.earth, NA)
		}
	}

	# Assembling the plot. 
	plot(0,0,type="n", xlab=xlab, ylab="", yaxt="n", xlim=c(start(incoming), end(incoming)), 
		ylim=c(length(ntracks)*-10, length(ptracks)*10), ...)
	text(mean(c(start(incoming), end(incoming))), length(ptracks)*5, "F", cex=5, col=rgb(0, 0, 0, 0.2))
	text(mean(c(start(incoming), end(incoming))), -length(ntracks)*5, "R", cex=5, col=rgb(0, 0, 0, 0.2))
	abline(h=0)
	if (!length(used.ex)) { return(invisible()) }

	# Drawing each track.	
    for (x in 1:length(used.ex)) {
        curx <- curex[[used.ex[x]]]
		isforward <- all(strand(curx)=="+")
		curtrack <- use.track[x]
		if (isforward) { 
			base.y <- (curtrack -1) * 10 + 2
			curcol <- fcol
		} else {
			base.y <- curtrack * -10 + 2
			curcol <- rcol
		}
		rect(start(curx), base.y, end(curx), base.y + 5, col=curcol, border=NA)

		# Drawing a line; using the middle so they don't overshoot the exon ends.
		allmiddies <- mid(ranges(curx))
		segments(min(allmiddies), base.y + 2.5, max(allmiddies), base.y + 2.5, col=curcol)

		# Pulling out the symbol.
		chosen <- match(used.ex[x], anno$gene_id)
		if (is.na(chosen)) { 
			myname <- paste0("ID:", used.ex[x])
		} else {
			myname <- anno$symbol[chosen]
		}
		text(middle.earth[x], base.y+5, myname, pos=3)
	}	
	invisible()	
}
