plotRegion <- function(cur.region, bam.file, param, max.depth=NULL, fcol="blue", rcol="red", 
	xlab="Genomic position (bp)", ylab="Read depth", ...)
# Exactly as specified. Takes a region and plots it in bimodal mode, with
# options for duplicate removal, mapping quality enhancement, colorization,
# and PET manipulation.
#
# written by Aaron Lun
{
	if (length(cur.region)!=1L) { stop("GRanges must be of length 1") } 
	cur.chr <- as.character(seqnames(cur.region))

	# Expanding the extracted region so that it's continuous past the plot margins.
	cur.width <- width(cur.region)
	max.len <- scanBamHeader(bam.file[1])[[1]]$targets[[cur.chr]]
	ext <- cur.width/2+1000 
	actual.region<-GRanges(cur.chr, IRanges(pmax(1L, start(cur.region)-ext), 
		pmin(max.len, end(cur.region)+ext)))
	reads <- extractReads(actual.region, bam.file, param=param)

	# Pulling out reads from a region and setting up coverage RLE's.
	if (param$pet!="both") {
		forward <- as.logical(strand(reads)=="+")
  		starts <- start(reads)
		ends <- end(reads)
	} else {
		# Plotting PETs with a bit more care.
		nx <- length(reads$pos)
		forward <- c(logical(nx), !logical(nx))
		midpoint <- as.integer(mid(ranges(reads)))
		starts <- c(midpoint+1L, start(reads))
 	    ends <- c(end(reads), midpoint)
	}
	pos<-coverage(IRanges(starts[forward], ends[forward]), width=max.len)
	neg<-coverage(IRanges(starts[!forward], ends[!forward]), width=max.len)
	
	# Just plotting.
	if (is.null(max.depth)) { max.depth<-max(runValue(pos), runValue(neg)) }
	ylim<-c(-max.depth, max.depth)
	plot(-1, 1, xlim=c(start(cur.region), end(cur.region)), ylim=ylim, type="n",
			xlab=xlab, ylab=ylab, ...)
	rect(start(pos), 0, c(start(pos)[-1], length(pos)), runValue(pos), col=fcol, border=NA)
	rect(start(neg), 0, c(start(neg)[-1], length(neg)), -runValue(neg), col=rcol, border=NA)

	return(invisible(NULL))
}

