loadRepeats <- function(dir)
# Load mask data from a nested directory of RepeatMasker files (constructed
# by detonating a tarbomb for the appropriate genome from UCSC). These can
# be obtained from 'http://hgdownload.cse.ucsc.edu/goldenPath/XXX/bigZips/'
# for a genome build 'XXX'.
#	
# written by Aaron Lun, 13/09/07
{
	everything<-list.files(dir, full.names=TRUE, recursive=TRUE, pattern=paste0("\\.fa\\.out$"))
	out<-list()
	for (x in 1:length(everything)) {
		data<-read.table(everything[x], sep="", skip=3, 
			colClasses=list(NULL, NULL, NULL, NULL, "character", "integer", "integer", 
					NULL, NULL, "character", NULL, NULL, NULL, NULL, NULL))
		y<-GRanges(data[,1], IRanges(data[,2], data[,3]))
		names(y)<-data[,4]
		out[[x]]<-y
	}
	suppressWarnings(out<-do.call(c, out))
	return(out)
}

pruneChromosomes <- function(chrs, mode=c("proper", "nuclear", "female", "autosomal"), pattern=FALSE)
# This function examines a set of chromosome names and pulls out only
# a subset; either only the properly names ones (i.e. chr...), 
# only the nuclear chromosomes, only the female subset or only 
# the autosomes.
{
	mode<-match.arg(mode)
	pat<-"^chr"	
	content <- ""
	if (mode!="proper") { 
		content<-"[0-9]+" 
		if (mode=="female") {
			content<-paste0(content, "|X")
		} else if (mode=="nuclear") {
			content<-paste0(content, "|X|Y")
		}
		pat <- paste0(pat, "(", content, ")$")
	}

	if (pattern) {  return(pat) }
	output<-logical(length(chrs))
	output[grep(pat, chrs)]<-TRUE
	return(output)	
}
