rematchPairs <- function(data, known) 
# This matches each index pair in 'known' to an observed bin pair in 'data',
# and reports the matching indices appropriately. Assumes that we have
# non-overlapping bin pairs and that only one chromosome is involved.
{
	storage.mode(known) <- "integer"
	ax <- anchors(data, id=TRUE)
	tx <- targets(data, id=TRUE)
	ds <- start(regions(data))
	de <- end(regions(data))
	if (length(runValue(seqnames(regions(data))))!=1L) { stop("only one chromosome supported") }

	od <- order(ax, tx)
	ax <- ax[od]
	tx <- tx[od]
	all.a <- rle(ax)
	a.vals <- all.a$values
	a.ends <- cumsum(all.a$lengths)
	a.starts <- c(0L, a.ends[-length(a.vals)])

	if (any(known[,1] < known[,2])) { stop("anchor indices should never be less than target indices") }
	out <- .Call("rematch_coords", known[,1], known[,2], 
			ds[a.vals], de[a.vals], a.starts, a.ends, 
			ds[tx], de[tx], PACKAGE="xscss")
	od[out]
}


