runDiffReps<-function(bam.files, grouping, tmpdir=tempfile(tmpdir="."), dpath="diffReps.pl", window=200, dedup=FALSE, 
		fraglen=100, threshold=NULL, mode="all")
# Run the DiffReps algorithm on a set of BAM files, given their differences. We need to convert
# them to BED files first (after removing any chromosomes we don't want) and then we run the main
# diffReps command, with specified options in terms of fragment lengths, window sizes and p-value
# thresholds.
{
    dir.create(tmpdir)
	all.bfs<-NULL
	for (bf in bam.files) {
		newfile<-file.path(tmpdir, paste0(basename(bf), ".bed"))
		convertBamToBed(bf, newfile, mode=mode, dedup=dedup)
		all.bfs<-append(all.bfs, newfile)
	}
	
	# Generating a genome file.
	genfile<-file.path(tmpdir, "gen.txt")
	chrs<-scanBamHeader(bam.files[1])[[1]][[1]]
	write.table(data.frame(names(chrs), chrs), file=genfile, col.names=FALSE, row.names=FALSE, sep="\t", quote=FALSE)
	
	# Generating the command string.
	grouping<-factor(grouping)
	is.first<-grouping==levels(grouping)[1]
	outfile<-file.path(tmpdir, "out.txt")
	cmds<-c(dpath, "--treatment", all.bfs[!is.first], "--control", all.bfs[is.first], "--nohs", "--noanno",
		"--chrlen", genfile, "--report", outfile, 
		"--window", round(window/10)*10, # Needs to be a multiple of 10.
		"--frag", fraglen)
	if (!is.null(threshold)) { cmds<-c(cmds, "--pval", threshold) }
	
	# Running the command, extracting and cleaning the results.
	if (system(paste(cmds, collapse=" "))) { stop("running diffReps failed") }
	obtained<-read.table(outfile, header=TRUE)
	if(nrow(obtained)) { 
		lcpm<-as.integer(NA) 
	} else { 
		lcpm<-integer(0) 
	}
	output<-data.frame(chr=obtained$Chrom, start=obtained$Start, end=obtained$End, 
			logFC=obtained$log2FC, logCPM=lcpm, PValue=obtained$pval, FDR=obtained$padj)
	unlink(tmpdir, recursive=TRUE)
	return(output)
}

