runMACS <- function(pooled, oprefix, macs.path="macs14", threshold=NULL, 
		gsize="mm", fraglen=NULL, cmd.only=FALSE, format="BED")
# Run MACS on a massive pooled BED file constructed from all the individual BAM files. We don't 
# allow duplicate removal as any duplicates should be removed prior to construction of the pooled 
# library. For example, you could get reads from separate libraries at the same position in the 
# pooled library.
{
	# Constructing the command string.
	cmd<-c(macs.path, "-t", pooled, paste0("--gsize=", gsize), paste0("--name=", oprefix), paste0("--format=", format), "--keep-dup=all")
	if (!is.null(fraglen)) { cmd<-c(cmd, " --nomodel", paste0("--shiftsize=", as.integer(fraglen/2+0.5))) }
	if (!is.null(threshold)) { cmd<-c(cmd, "-p", threshold) }

	# Running MACS. 
	if (cmd.only) { return(paste(cmd, collapse=" ")) }
	if (system(paste(cmd, collapse=" "))) { stop("running MACS failed") }
	return(invisible(NULL))
}

runMACS2 <- function(file, outprefix, macs.path="macs2", threshold=NULL,
		gsize="mm", fraglen=NULL, cmd.only=FALSE, extra=NULL, format="BED") 
# This does much the same, with the new MACS2 peak caller. 
{
	cmds <- c(macs.path, "callpeak -t", file, "--gsize", gsize, "--keep-dup=all", "-f", format,
			"--outdir", dirname(outprefix), "-n", basename(outprefix), extra)
	if (!is.null(fraglen)) { cmds <- c(cmds, "--nomodel --extsize", fraglen) }
	if (!is.null(threshold)) { cmds <- c(cmds, "-p", threshold) }
	
	if (cmd.only) { return(paste(cmds, collapse=" ")) }
	if (system(paste(cmds, collapse=" "))) { stop("running MACS2 failed") }
	return(invisible(NULL))
}

###########################
#	# SISSRs, running on said BED file. I only keep this for reference purposes.
#	if (!is.null(sissrs.path)) { 
#		outtemp<-file.path(peakdir, .sissrsfile());
#		cmd<-paste(sissrs.path, "-i", outfile, "-o", outtemp, "-s", gsize)
#		system(cmd)
#		newtemp<-file.path(peakdir, "out2.txt")
#		system(paste("cat", outtemp, "| grep '^chr' >", newtemp))
#		file.rename(newtemp, outtemp)
#	}
###########################

loadMACS <- function(oprefix, summit=FALSE, negative=FALSE, threshold=NULL, use.new=FALSE)
# Load a MACS output file into existence.
{
	candidates<-read.table(paste0(oprefix, ifelse(negative, "_negative", ""), "_peaks.xls"), header=TRUE)
	if (!summit) {
		mregions<-GRanges(candidates$chr, IRanges(candidates$start, candidates$end))
	} else {
		summit.pos<-candidates$summit+candidates$start
		mregions<-GRanges(candidates$chr, IRanges(summit.pos, summit.pos));
	}
	if (!is.null(threshold)) {
 	    pcol <- 7	
		pattern <- ifelse(use.new, "X-log10(pvalue)", "X-10*log10(pvalue)")
		if (!length(grep(colnames(candidates)[pcol], pattern))) {
			warning(paste0("designated column (", colnames(candidates)[pcol], 
				") may not contain the transformed p-value")) 
		}
		if (is.na(threshold)) { 
			mregions<-list(region=mregions, pval=candidates[,pcol])
		} else {
   			mregions<-mregions[candidates[,pcol]>=threshold,] 
		}
	}
	return(mregions)
}

#############################################################################


