peakFile <- function(fname, chrs, pos, mus, disp, sizes, fraglen=100, width=1000, rlen=10, tf=TRUE, append=FALSE, alpha=2, beta=2) 
# Spiking in peaks more manually.
{
	if (!append) { .addHeader(fname, sizes) }

	pos<-as.integer(pos+0.5)
	y<-as.integer(rnbinom(length(pos), mu=mus, size=1/disp)+0.5)
	fraglen<-as.integer(fraglen+0.5)
	rlen<-as.integer(rlen+0.5)
	width<-as.integer(width+0.5)
	storage.mode(sizes)<-"integer"

	for (chr in unique(chrs)) {
		chosen<-chrs==chr
		if (tf) {
			out<-.Call("R_dump_tf", fname, chr, pos[chosen], y[chosen], fraglen, rlen, sizes[[chr]], alpha, beta, PACKAGE="xscss")
		} else {
			out<-.Call("R_dump_histone", fname, chr, pos[chosen], y[chosen], width, rlen, sizes[[chr]], alpha, beta, PACKAGE="xscss")
		}
		if (is.character(out)) { stop(out) }
	}
	return(invisible(NULL))
}

bgFile <- function(fname, chrs, mus, disp, sizes, rlen=10, width=2000, append=FALSE)
# Spiking in background manually.
{
	if (!append) { .addHeader(fname, sizes) }
	width <- as.integer(width + 0.5)
	rlen <- as.integer(rlen + 0.5)

	for (x in names(sizes)) {
		chosen <- chrs==x
		nbins <- sum(chosen)
		if (nbins*width > sizes[[x]]) { stop("number of bins exceeds width of chromosome") }
		y <- as.integer(rnbinom(nbins, mu=mus[chosen], size=1/disp[chosen]))
		out <- .Call("R_dump_background", fname, x, width, y, rlen, PACKAGE="xscss")
		if (is.character(out)) { stop(out) }
	}
	
	return(invisible())	
}

addBackground <- function(fnames, sizes, rlen=10, width=2000, back.mu=c(10, 50), prior.df=20, dispersion=0.05, append=FALSE) 
# Adding the same background regions to all files.
{
	if (!append) {
		for (fname in fnames) { .addHeader(fname, sizes) }
	}
	
	# Setting integers.
	width<-as.integer(width+0.5)
	rlen<-as.integer(rlen+0.5)

	# Setting up the background.
	for (x in names(sizes)) {
		nbins<-as.integer(sizes[[x]]/width)
		bmu<-runif(nbins, back.mu[1], back.mu[2])
		bdisp<-dispersion*prior.df/rchisq(nbins, df=prior.df)
		for (fname in fnames) {
			back.y<-as.integer(rnbinom(nbins, mu=bmu, size=1/bdisp)+0.5)
		    out<-.Call("R_dump_background", fname, x, width, back.y, rlen, PACKAGE="xscss")
			if (is.character(out)) { stop(out) }
		}
	}
	return(invisible())
}

.addHeader <- function(fname, sizes) {
	write.table(file=fname, data.frame("@SQ", paste("SN", names(sizes), sep=":"),
		paste("LN", as.integer(sizes), sep=":")), quote=FALSE, row.names=FALSE, sep="\t", col.names=FALSE)
	invisible(NULL)
}
