#include "R.h"
#include "Rdefines.h"

SEXP expand_coords(SEXP mat, SEXP nrow, SEXP width, SEXP len) 
/* For use in expandLigations. */
{
	int w=INTEGER_VALUE(width), 
		l=INTEGER_VALUE(len),
		nlen=INTEGER_VALUE(nrow);
	int * aptr=INTEGER_POINTER(mat),
	    * tptr=aptr + nlen;

	SEXP output=PROTECT(NEW_LIST(nlen));
	int ix, ax, tx,
		uppera, lowera, 
		uppert, lowert, 
		maxsize, diff;
	int * oaptr, 
		* otptr;

	/* Setting up names. */
	SEXP dimnames = PROTECT(allocVector(VECSXP, 2));
	SET_VECTOR_ELT(dimnames, 0, R_NilValue);
	SET_VECTOR_ELT(dimnames, 1, NEW_CHARACTER(2));
	SEXP mynames = VECTOR_ELT(dimnames, 1);
	SET_STRING_ELT(mynames, 0, mkChar("anchor"));
	SET_STRING_ELT(mynames, 1, mkChar("target"));

	for (ix=0; ix<nlen; ++ix) {
		uppera=aptr[ix]+w;
		if (uppera > l) { uppera=l; }
		lowera=aptr[ix]-w;
		if (lowera < 1) { lowera=1; }
		uppert=tptr[ix]+w;
		if (uppert > l) { uppert=l; }
		lowert=tptr[ix]-w;
		if (lowert < 1) { lowert=1; }
		if (aptr[ix] < tptr[ix] ) {
			UNPROTECT(1);
			return mkString("anchor indices should be less than target indices in matrix of coordinates");
		}

		/* Finding the actual number of points. */
		maxsize=(uppera-lowera+1)*(uppert-lowert+1);
		diff=uppert-lowera;
		if (diff > 0) { maxsize -= diff*(diff+1)/2; }
		SET_VECTOR_ELT(output, ix, allocMatrix(INTSXP, maxsize, 2));

		/* Filling in the matrix with the new coordinates. */
		oaptr=INTEGER_POINTER(VECTOR_ELT(output, ix));
		otptr=oaptr+maxsize;
		for (ax=lowera; ax <= uppera; ++ax) {
			for (tx=lowert; tx <= ax && tx <= uppert; ++tx) {
				*(oaptr)=ax;
				*(otptr)=tx;
				++oaptr;
				++otptr;
			}
		}
	    setAttrib(VECTOR_ELT(output, ix), R_DimNamesSymbol, dimnames);
	}

	UNPROTECT(2);
	return output;
}
