#include "R.h"
#include "Rinternals.h"
#include <algorithm>

extern "C" {

SEXP rematch_coords(SEXP anchor_inter, SEXP target_inter, 
		SEXP astart, SEXP aend, SEXP arstart, SEXP arend,
		SEXP tstart, SEXP tend) {
	int * aiptr=INTEGER(anchor_inter),
		* tiptr=INTEGER(target_inter),
		* asptr=INTEGER(astart),
		* aeptr=INTEGER(aend),
		* rsptr=INTEGER(arstart),
		* reptr=INTEGER(arend),
		* tsptr=INTEGER(tstart),
		* teptr=INTEGER(tend);
	int numinter=LENGTH(anchor_inter), 
		numrun=LENGTH(astart),
		numpair=LENGTH(tstart);
	
	SEXP output=PROTECT(allocVector(INTSXP, numinter));
	int * optr=INTEGER(output);
	int temp_a, temp_t;
	int * aefinal=aeptr+numrun,
	    * testart, * tefinal;

	for (int interx=0; interx<numinter; ++interx) {
		/* Binary search to find the anchor in which this interaction lies. */
		temp_a=std::lower_bound(aeptr, aefinal, aiptr[interx])-aeptr;
		if (temp_a==numrun || asptr[temp_a] > aiptr[interx]) {
			optr[interx]=NA_INTEGER;
			continue;			
		}

		/* Binary search to find the target corresponding to this interaction.  */
		testart=teptr+rsptr[temp_a];
		tefinal=teptr+reptr[temp_a];
		temp_t=std::lower_bound(testart, tefinal, tiptr[interx])-teptr;
		if (temp_t==reptr[temp_a] || tsptr[temp_t] > tiptr[interx]) {
			optr[interx]=NA_INTEGER;
			continue;
		}	
		
		/* Storing the matching index. */
		optr[interx]=temp_t+1;
	}

	UNPROTECT(1);
	return output;	
}

}
