####################################################################################################
# Tests the assessChIP function, and makes sure that it behaves properly.

suppressPackageStartupMessages(require(xscss))
suppressPackageStartupMessages(require(csaw))
odir<-"temp"
dir.create(odir)

asscomp <- function(tol=200, checkfc=TRUE) {
	# Generate a random set of genomic ranges.
	chrs<-c("chrA","chrB", "chrX")
	all.chrs<-all.pos<-all.ends<-all.fc<-NULL
	fullsize<-integer(0)
	for (x in chrs) {
		n<-1000
		all.chrs<-append(all.chrs, rep(x, n))
		cursizes<-round(runif(n, 100, 1000))
		curspace<-round(runif(n, 100, 1000))
		curends<-cumsum(cursizes+curspace)-1L
		curstarts<-curends-cursizes+1L
		all.pos<-append(all.pos, curstarts)
		all.ends<-append(all.ends, curends)
		all.fc<-append(all.fc, cumprod(ifelse(rbinom(n, 1, 0.9)==1, 1, -1)))
		fullsize[[x]]<-max(curends)
	}

	depo<-file.path(odir, "stuff.tsv")
	write.table(file=depo, data.frame(chr=all.chrs, start=all.pos, end=all.ends, logFC=all.fc),
		quote=FALSE, sep="\t", row.names=FALSE)

	# Making a set of reference ranges.
	m<-50
	ref.chrs<-sample(chrs, m, replace=TRUE)
	matched<-match(chrs, names(fullsize))
	ref.start<-round(runif(m, 1, fullsize[matched]))
	ref.end<-round(runif(m, 1000, 10000))+ref.start
	ref.fc<-sample(c(-1, 1), m, replace=TRUE)
	refdeo<-file.path(odir, "ref.tsv")
	write.table(file=refdeo, data.frame(chr=ref.chrs, start=ref.start, end=ref.end, logFC=ref.fc),
		quote=FALSE, sep="\t", row.names=FALSE)

	# Checks the assessment machinery, using... well, almost the same code.
	if (!checkfc) { 
		ref.fc<-rep(1, length(ref.fc))
		all.fc<-rep(1, length(all.fc))
	}
	alpha<-GRanges(all.chrs, IRanges(all.pos, all.ends))
	if (!is.na(tol)) { 
		realpha<-mergeWindows(alpha, tol=tol, sign=all.fc>0)
		alpha<-realpha$region
		all.fc<-aggregate(all.fc~realpha$id, FUN=head, data=NULL, n=1)$all.fc
	}
	bravo<-GRanges(ref.chrs, IRanges(ref.start, ref.end))

	uplap<-findOverlaps(bravo[ref.fc>0], alpha[all.fc>0])
	dwlap<-findOverlaps(bravo[ref.fc<0], alpha[all.fc<0])
	obs<-assessChIP(depo, refdeo, tol=tol, checkfc=checkfc)
	if (!identical(length(alpha), obs$found)) { stop("total number of found regions are non-equal") }
	if (!identical(length(unique(queryHits(uplap)))+length(unique(queryHits(dwlap))), obs$recall)) { stop("total recall is non equal") }
	if (!identical(length(unique(subjectHits(uplap)))+length(unique(subjectHits(dwlap))), obs$overlap)) { stop("total overlap is non equal") }
	return(obs)
}

set.seed(2198910)
asscomp(100, TRUE)

asscomp(200, TRUE)

asscomp(500, TRUE)

asscomp(1000, TRUE)

asscomp(100, FALSE)

asscomp(200, FALSE)

asscomp(500, FALSE)

asscomp(1000, FALSE)

unlink(odir, recursive=TRUE)
