####################################################################################################
# This tests whether the sorting and indexing behaviour in crunch2BAM actually does its job.

suppressPackageStartupMessages(require(xscss))
suppressPackageStartupMessages(require(Rsamtools))

odir<-"temp"
dir.create(odir)
fname<-file.path(odir, "out.sam")

autocomp <- function() {
   	n<-10
 	sizes<-rep(10000, 6)
	names(sizes)<-c("chr1", "chr21", "chrM", "chrX", "chrY", "chr12_random")
	chrs<-sample(names(sizes), n, replace=T)
	mu<-rep(20, n)
	pos<-round(runif(n, 2000, 8000))
	disp<-rep(0.05, n)
	peakFile(fname, chrs=chrs, pos=pos, mu=mu, disp=disp, sizes=sizes)

	bam<-crunch2BAM(fname, dir=odir)
	if (!file.exists(bam) || !file.exists(paste0(bam, ".bai"))) { stop("conversion to BAM failed") }
	blah<-scanBam(bam)[[1]]
	# First checking that everyone is, indeed, sorted.
	x<-rle(as.character(blah$rname))
	if (length(unique(x$values))!=length(x$values)) { stop("reads are not sorted by chromosome")  }
	gunk<-split(blah$pos, blah$rname)
	if (any(sapply(gunk, FUN=is.unsorted))) { stop("reads are not sorted by position in each chromosome") }
	
	# Checking consistency between names and flags.
	blah2<-read.table(fname, comment="@",stringsAsFactors=FALSE)
	ro<-match(blah2[,1], blah$qname)
	if (length(blah$qname)!=nrow(blah2) || any(is.na(ro))) { stop("missing reads") }
	if (!identical(blah$flag[ro], blah2[,2])) { stop("flags don't match up") }
	return(head(blah2))
}

set.seed(847548)
autocomp()

autocomp()

autocomp()

autocomp()

unlink(odir, recursive=TRUE)

