# This tests the expandLigations function in simHic.

suppressPackageStartupMessages(require(simHic))
suppressPackageStartupMessages(require(diffHic))

comparator <- function(mat, width, chrlen) {
	ref <- expandLigations(mat, width, chrlen)

	for (x in 1:nrow(mat)) { 
		current <- ref[[x]]
		all.a <- max(1L, mat[x,1] - width):min(chrlen, mat[x,1]+width)
		all.t <- max(1L, mat[x,2] - width):min(chrlen, mat[x,2]+width)
		comparison <- expand.grid(all.a, all.t)
		keep <- comparison[,1] >= comparison[,2]
		comparison <- as.matrix(comparison[keep,,drop=FALSE])
		o <- order(comparison[,1], comparison[,2])
		comparison <- comparison[o,,drop=FALSE]
		colnames(comparison) <- c("anchor", "target")
		rownames(comparison) <- NULL
		if (!identical(comparison, current)) { stop("mismatch between expanded ligations") }
	}

	return(head(ref[[1]]))
}


generateKnown <- function(chrlen, npts) {
	ax <- sample(chrlen, npts)
	tx <- sample(chrlen, npts)
	cbind(pmax(ax, tx), pmin(ax, tx))
}

##########################################################

set.seed(23478283)

known <- generateKnown(1000, 50)
comparator(known, 0, 1000)
comparator(known, 1, 1000)
comparator(known, 2, 1000)
comparator(known, 5, 1000)

known <- generateKnown(1000, 100)
comparator(known, 0, 1000)
comparator(known, 1, 1000)
comparator(known, 2, 1000)
comparator(known, 5, 1000)

known <- generateKnown(500, 20)
comparator(known, 0, 1000)
comparator(known, 1, 1000)
comparator(known, 2, 1000)
comparator(known, 5, 1000)

##########################################################
# End.

