# This tests the behaviour of the rematchPairs utility.

suppressPackageStartupMessages(require(simHic))
suppressPackageStartupMessages(require(diffHic))

comparator <- function(data, known) {
	output <- as.integer(rep(NA, nrow(known)))
	for (x in 1:nrow(known)) { 
		alap <- findOverlaps(ranges(anchors(data)), IRanges(known[x,1], known[x,1]))
		tlap <- findOverlaps(ranges(targets(data)), IRanges(known[x,2], known[x,2]))
		present <- intersect(queryHits(alap), queryHits(tlap))
		if (length(present)==2L) { stop("multiple indices on ", x) }
		if (any(present)) { output[x] <- present }
	}

	ref <- rematchPairs(data, known)
	if (!identical(ref, output)) { 
		stop("mismatch between nested identities")
	}
	return(head(ref))
}

generateData <- function(nbins, chrlen, binsize) {
	starts <- seq(1, chrlen, by=binsize)
	ends <- pmin(starts+binsize - 1L, chrlen)
	regions <- GRanges("chr", IRanges(starts, ends))

	bin.num <- length(regions)
	inter.space <- bin.num*(bin.num+1)/2
	chosen <- sample(inter.space, nbins)
	anchor <- ceiling((-1 + sqrt(1+8*chosen))/2)
	target <- chosen - anchor*(anchor-1)/2 

	DIList(cbind(integer(nbins)), anchors=anchor, targets=target, regions=regions)
}

generateKnown <- function(chrlen, npts) {
	ax <- sample(chrlen, npts)
 	tx <- sample(chrlen, npts)
	cbind(pmax(ax, tx), pmin(ax, tx))
}

##########################################################

set.seed(3457349)

data <- generateData(500, 1000, 20)
known <- generateKnown(1000, 100)
comparator(data, known)

data <- generateData(500, 1000, 23)
known <- generateKnown(1000, 100)
comparator(data, known)

data <- generateData(100, 1000, 50)
known <- generateKnown(1000, 100)
comparator(data, known)

data <- generateData(100, 1000, 59)
known <- generateKnown(1000, 100)
comparator(data, known)

data <- generateData(50, 1000, 100)
known <- generateKnown(1000, 100)
comparator(data, known)

data <- generateData(50, 1000, 104)
known <- generateKnown(1000, 100)
comparator(data, known)

##########################################################
# End.

